#
# Copyright 2007 Fedora Unity
#
# Jonathan Steffan <jon a fedoraunity.org>
# Jeroen van Meeuwen <kanarip a fedoraunity.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 only
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import os
import shutil
import sys
import subprocess

# Translation
from rhpl.translate import _, N_, getDefaultLangs
import rhpl.translate as translate

class RevisorRebrand:
    def __init__(self, base):
        """Builds rebranded packages"""
        self.base = base
        self.cfg = base.cfg
        self.log = base.log

    def build_specs(self):
        pass

    def build_makefiles(self):
        pass

    def build_packages(self):
        """Build rebranded packages from a directory tree:
        - %name-release,
        - %name-release-notes,
        - %name-logos
        using a (base) directory tree (self.cfg.rebrand_directory) in which all appropriate files and directories exist,
        a template directory (which we have)
        """

        # Directory tree is at self.cfg.rebrand_directory

        self.build_package_release()
        self.build_package_release_notes()
        self.build_package_logos()

    def build_package_logos(self):
        # Check if os.path.join(self.cfg.rebrand_directory,"%s-logos" % self.cfg.product_name) exists
        # If it does, try modifying the Makefile and .spec files
        # If it doesn't, copy over the template directory and edit it
        pass

    def build_package_release_notes(self):
        # Check if os.path.join(self.cfg.rebrand_directory,"%s-release" % self.cfg.product_name) exists
        # If it does, try modifying the Makefile and .spec files
        # If it doesn't, copy over the template directory and edit it
        self.log.debug(_("Check if os.path.join(self.cfg.rebrand_directory,\"%s-logos\" % self.cfg.product_name) exists"))

        # No matter how things go, the Makefile and specfile should be outputted here:
        specfile_output = os.path.join(self.cfg.rebrand_directory,"%s-logos-%s" % (self.cfg.product_name,self.cfg.version),"%s-logos.spec" % self.cfg.product_name)
        makefile_output = os.path.join(self.cfg.rebrand_directory,"%s-logos-%s" % (self.cfg.product_name,self.cfg.version),"Makefile")

        # The question is though, what is our makefile_input and specfile_input?
        specfile_input = ""
        makefile_input = ""

        # The destination directory is _always_ the same:
        dst = os.path.join(self.cfg.rebrand_directory,"%s-logos-%s" % (self.cfg.product_name,self.cfg.version))

        if os.access(os.path.join(self.cfg.rebrand_directory,"%s-logos-%s" % (self.cfg.product_name,self.cfg.version)), os.R_OK):
            # Someone has done some preparation
            # Let's see if they have a proper specfile_input also
            if os.access(os.path.join(dst,"%s-logos.spec" % self.cfg.product_name), os.R_OK):
                specfile_input = os.path.join(dst,"%s-logos.spec" % self.cfg.product_name)
            if os.access(os.path.join(dst,"Makefile"), os.R_OK):
                makefile_input = os.path.join(dst,"Makefile")
        elif os.access(os.path.join(self.cfg.rebrand_directory,"%s-logos" % (self.cfg.product_name)), os.R_OK):
            # Someone has done /some/ preparation, but hasn't put the version number in the directory name
            src = os.path.join(self.cfg.rebrand_directory,"%s-logos" % (self.cfg.product_name))
            self.log.debug(_("Copying %s to %s") % (src,dst))
            shutil.copytree(src,dst)

            # Let's see if they have a proper specfile_input also
            if os.access(os.path.join(dst,"%s-logos.spec" % self.cfg.product_name), os.R_OK):
                specfile_input = os.path.join(dst,"%s-logos.spec" % self.cfg.product_name)
            if os.access(os.path.join(dst,"Makefile"), os.R_OK):
                makefile_input = os.path.join(dst,"Makefile")
        else:
            # No preparation whatsoever
            src = "/usr/share/revisor/rebrand-template/template-logos/"
            self.log.debug(_("Copying %s to %s") % (src,dst))
            shutil.copytree(src,dst)

            # That shouldn't have a proper spec file but you never know
            if os.access(os.path.join(dst,"%s-logos.spec" % self.cfg.product_name), os.R_OK):
                specfile_input = os.path.join(dst,"%s-logos.spec" % self.cfg.product_name)
            if os.access(os.path.join(dst,"Makefile"), os.R_OK):
                makefile_input = os.path.join(dst,"Makefile")

        # If we have no specfile now, we use the template
        if specfile_input == "":
            specfile_input = "/usr/share/revisor/rebrand-template/template-logos/template-logos.spec.tpl"
        if makefile_input == "":
            makefile_input = "/usr/share/revisor/rebrand-template/template-logos/Makefile.tpl"

        specf = open(specfile_input, "r")
        specin = specf.read()
        specf.close()

        specout = specin.replace("@version@",self.cfg.version)
        specout = specout.replace("@product_name@", self.cfg.product_name)
        specout = specout.replace("@release@",self.cfg.release)
        specout = specout.replace("@release_codename@",self.cfg.release_codename)
        specout = specout.replace("@release_url@",self.cfg.release_url)

        specf = open(specfile_output, "w")
        specf.write(specout)
        specf.close()

        makef = open(makefile_input, "r")
        makein = makef.read()
        makef.close()

        makeout = makein.replace("@version@",self.cfg.version)
        makeout = makeout.replace("@product_name@", self.cfg.product_name)
        makeout = makeout.replace("@release@",self.cfg.release)
        makeout = makeout.replace("@release_codename@",self.cfg.release_codename)
        makeout = makeout.replace("@release_url@",self.cfg.release_url)

        makef = open(makefile_output, "w")
        makef.write(makeout)
        makef.close()

        # Now that we have a spec and Makefile in place, let's go build
        # in dst, execute "make"
        callback = MakeCallback()
        self.base.run_command(["make"],rundir=dst,callback=callback)

    def build_package_release(self):
        # Check if os.path.join(self.cfg.rebrand_directory,"%s-release" % self.cfg.product_name) exists
        # If it does, try modifying the Makefile and .spec files
        # If it doesn't, copy over the template directory and edit it
        self.log.debug(_("Check if os.path.join(self.cfg.rebrand_directory,\"%s-release\" % self.cfg.product_name) exists"))

        # No matter how things go, the Makefile and specfile should be outputted here:
        specfile_output = os.path.join(self.cfg.rebrand_directory,"%s-release-%s" % (self.cfg.product_name,self.cfg.version),"%s-release.spec" % self.cfg.product_name)
        makefile_output = os.path.join(self.cfg.rebrand_directory,"%s-release-%s" % (self.cfg.product_name,self.cfg.version),"Makefile")

        # The question is though, what is our makefile_input and specfile_input?
        specfile_input = ""
        makefile_input = ""

        # The destination directory is _always_ the same:
        dst = os.path.join(self.cfg.rebrand_directory,"%s-release-%s" % (self.cfg.product_name,self.cfg.version))

        if os.access(os.path.join(self.cfg.rebrand_directory,"%s-release-%s" % (self.cfg.product_name,self.cfg.version)), os.R_OK):
            # Someone has done some preparation
            # Let's see if they have a proper specfile_input also
            if os.access(os.path.join(dst,"%s-release.spec" % self.cfg.product_name), os.R_OK):
                specfile_input = os.path.join(dst,"%s-release.spec" % self.cfg.product_name)
            if os.access(os.path.join(dst,"Makefile"), os.R_OK):
                makefile_input = os.path.join(dst,"Makefile")
        elif os.access(os.path.join(self.cfg.rebrand_directory,"%s-release" % (self.cfg.product_name)), os.R_OK):
            # Someone has done /some/ preparation, but hasn't put the version number in the directory name
            src = os.path.join(self.cfg.rebrand_directory,"%s-release" % (self.cfg.product_name))
            self.log.debug(_("Copying %s to %s") % (src,dst))
            shutil.copytree(src,dst)

            # Let's see if they have a proper specfile_input also
            if os.access(os.path.join(dst,"%s-release.spec" % self.cfg.product_name), os.R_OK):
                specfile_input = os.path.join(dst,"%s-release.spec" % self.cfg.product_name)
            if os.access(os.path.join(dst,"Makefile"), os.R_OK):
                makefile_input = os.path.join(dst,"Makefile")
        else:
            # No preparation whatsoever
            src = "/usr/share/revisor/rebrand-template/template-release/"
            self.log.debug(_("Copying %s to %s") % (src,dst))
            shutil.copytree(src,dst)

            # That shouldn't have a proper spec file but you never know
            if os.access(os.path.join(dst,"%s-release.spec" % self.cfg.product_name), os.R_OK):
                specfile_input = os.path.join(dst,"%s-release.spec" % self.cfg.product_name)
            if os.access(os.path.join(dst,"Makefile"), os.R_OK):
                makefile_input = os.path.join(dst,"Makefile")

        # If we have no specfile now, we use the template
        if specfile_input == "":
            specfile_input = "/usr/share/revisor/rebrand-template/template-release/template-release.spec.tpl"
        if makefile_input == "":
            makefile_input = "/usr/share/revisor/rebrand-template/template-release/Makefile.tpl"

        specf = open(specfile_input, "r")
        specin = specf.read()
        specf.close()

        specout = specin.replace("@version@",self.cfg.version)
        specout = specout.replace("@product_name@", self.cfg.product_name)
        specout = specout.replace("@release@",self.cfg.release)
        specout = specout.replace("@release_codename@",self.cfg.release_codename)
        specout = specout.replace("@release_url@",self.cfg.release_url)

        specf = open(specfile_output, "w")
        specf.write(specout)
        specf.close()

        makef = open(makefile_input, "r")
        makein = makef.read()
        makef.close()

        makeout = makein.replace("@version@",self.cfg.version)
        makeout = makeout.replace("@product_name@", self.cfg.product_name)
        makeout = makeout.replace("@release@",self.cfg.release)
        makeout = makeout.replace("@release_codename@",self.cfg.release_codename)
        makeout = makeout.replace("@release_url@",self.cfg.release_url)

        makef = open(makefile_output, "w")
        makef.write(makeout)
        makef.close()

        # Now that we have a spec and Makefile in place, let's go build
        # in dst, execute "make"
        callback = MakeCallback()
        self.base.run_command(["make"],rundir=dst,callback=callback)

class MakeCallback:
    def __init__(self):
        pass

    def parse_line(self, msg):
        print msg
